import os
import logging
import time

from constants import SA_WEB_SERVICES_HOST
from constants import NAMESPACE
from constants import UI_POD_LABEL_SELECTOR
from constants import WEB_SERVICE_POD_LABEL_SELECTOR
from constants import RETRY_COUNT
from constants import Url
from utils import requests_wrapper
from kubernetes import client, config


def wait_for_pod_readiness(namespace, label_selector):
    config.load_incluster_config()
    v1 = client.CoreV1Api()
    while True:
        try:
            logging.debug("Read pod namespace")
            pods = v1.list_namespaced_pod(NAMESPACE, label_selector=label_selector)
            pod_name = None
            if pods.items is not None and len(pods.items) > 0:
                # As of now, we have only 1 pod for malware-prevention-ui and sa-web-services. Hence, getting the
                # first item.
                first_item = pods.items[0]
                pod_name = first_item.metadata.name
                logging.debug(pod_name)
            else:
                logging.info("Pod list is either None or empty, retrying in 10 seconds.")
                time.sleep(10)
                continue
            pod = v1.read_namespaced_pod(name=pod_name, namespace=namespace)
            phase = pod.status.phase
            logging.debug("%s pod status %s", pod_name, phase)
            if phase.lower() == "running" and all(container.ready for container in pod.status.container_statuses):
                logging.info("Pod : %s is ready", pod_name)
                return
        except Exception as e:
            logging.exception(e)
            raise
        logging.info("Pod %s not ready, retrying in 10 seconds...", pod_name)
        time.sleep(10)


def sync_system_svm_specs(retry=None):
    if retry is None:
        retry = RETRY_COUNT
    try:
        sync_system_svm_request_url = Url.InCluster.SYNC_SYSTEM_SVM_OVF.format(
            service_host=SA_WEB_SERVICES_HOST)
        logging.info("Calling Malware Prevention Sync System SVM Spec API")
        requests_wrapper.incluster_request('POST', sync_system_svm_request_url)
    except Exception as e:
        if retry == 1:
            raise e
        logging.exception(e)
        logging.error("Register SVM failed. Hence retrying")
        sync_system_svm_specs(retry - 1)


def validate_nsx_version():
    logging.info("Get NSX Version")
    try:
        r = requests_wrapper.egress_request('GET', Url.Egress.GET_NSX_VERSION)
        response_json = r.json()
        version = response_json['product_version']  # "product_version": "4.2.0.0.0.23423638"
        logging.info("NSX Version is : %s", version)
        parts = version.split('.')
        if len(parts) >= 2:
            major_version = int(parts[0])
            minor_version = int(parts[1])
            # For any NSX Version which is less than 4.2, we should not call the SVM registration API
            if major_version < 4 or (major_version == 4 and minor_version < 2):
                return False
            else:
                logging.info("NSX version is valid")
                return True
        else:
            return False
    except Exception as e:
        logging.exception(e)
        raise


def create_service_definition():
    if not validate_nsx_version():
        logging.info("NSX version is not valid. Hence, not creating SVM spec.")
        return
    logging.info("Initial sleep for 120 seconds")
    time.sleep(120)
    chart_version = os.environ["CHART_VERSION"]
    logging.info("Chart version : %s", chart_version)
    try:
        wait_for_pod_readiness(NAMESPACE, UI_POD_LABEL_SELECTOR.format(chart_version=chart_version))
        logging.info("Malware Prevention UI Pod is ready.")
        wait_for_pod_readiness(NAMESPACE, WEB_SERVICE_POD_LABEL_SELECTOR.format(chart_version=chart_version))
        logging.info("sa-web-service pod is ready. Making API call to register SVM.")
        sync_system_svm_specs(RETRY_COUNT)
    except Exception as e:
        logging.exception(e)
        logging.error("MPS SVM registration failed.")
